@@ -42,6 +42,8 @@ const (
 	RouterUniqueHeaderName   = "ROUTER_UNIQUE_ID_HEADER_NAME"
 	RouterUniqueHeaderFormat = "ROUTER_UNIQUE_ID_FORMAT"
 
+	RouterHTTPHeaderNameCaseAdjustments = "ROUTER_H1_CASE_ADJUST"
+
 	RouterLogLevelEnvName       = "ROUTER_LOG_LEVEL"
 	RouterSyslogAddressEnvName  = "ROUTER_SYSLOG_ADDRESS"
 	RouterSyslogFormatEnvName   = "ROUTER_SYSLOG_FORMAT"
@@ -56,6 +58,8 @@ const (
 
 	RouterHardStopAfterEnvName    = "ROUTER_HARD_STOP_AFTER"
 	RouterHardStopAfterAnnotation = "ingress.operator.openshift.io/hard-stop-after"
+
+	LivenessGracePeriodSecondsAnnotation = "unsupported.do-not-use.openshift.io/override-liveness-grace-period-seconds"
 )
 
 // ensureRouterDeployment ensures the router deployment exists for a given
@@ -73,7 +77,7 @@ func (r *reconciler) ensureRouterDeployment(ci *operatorv1.IngressController, in
 	if err != nil {
 		return false, nil, fmt.Errorf("failed to determine if proxy protocol is needed for ingresscontroller %s/%s: %v", ci.Namespace, ci.Name, err)
 	}
-	desired, err := desiredRouterDeployment(ci, r.Config.IngressControllerImage, ingressConfig, apiConfig, networkConfig, proxyNeeded)
+	desired, err := desiredRouterDeployment(ci, r.config.IngressControllerImage, ingressConfig, apiConfig, networkConfig, proxyNeeded)
 	if err != nil {
 		return haveDepl, current, fmt.Errorf("failed to build router deployment: %v", err)
 	}
@@ -220,7 +224,7 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		// HostNetwork.
 	case operatorv1.PrivateStrategyType, operatorv1.LoadBalancerServiceStrategyType, operatorv1.NodePortServiceStrategyType:
 		// To avoid downtime during a rolling update, we need two
-		// things: a deployment strategy and affinity policy.  First,
+		// things: a deployment strategy and an affinity policy.  First,
 		// the deployment strategy: During a rolling update, we want the
 		// deployment controller to scale up the new replica set first
 		// and scale down the old replica set once the new replica is
@@ -247,16 +251,16 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		// pod is colocated with a pod from the old replica set.  To
 		// this end, we add a label with a hash of the deployment, using
 		// which we can select replicas of the same generation (or
-		// select replicas that are *not* of the same generation),
-		// configure affinity to colocate replicas of different
-		// generations of the same ingress controller, and configure
+		// select replicas that are *not* of the same generation).
+		// Then, we can configure affinity to colocate replicas of
+		// different generations of the same ingress controller, and configure
 		// anti-affinity to prevent colocation of replicas of the same
 		// generation of the same ingress controller.
 		//
 		// Together, the deployment strategy and affinity policy ensure
 		// that a node that had local endpoints at the start of a
 		// rolling update continues to have local endpoints for the
-		// throughout and at the completion of the update.
+		// duration of and at the completion of the update.
 		needDeploymentHash = true
 		deployment.Spec.Template.Spec.Affinity = &corev1.Affinity{
 			PodAffinity: &corev1.PodAffinity{
@@ -405,6 +409,7 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	}
 
 	deployment.Spec.Template.Spec.Containers[0].Image = ingressControllerImage
+	deployment.Spec.Template.Spec.DNSPolicy = corev1.DNSClusterFirst
 
 	if ci.Status.EndpointPublishingStrategy.Type == operatorv1.HostNetworkStrategyType {
 		// Expose ports 80 and 443 on the host to provide endpoints for
@@ -417,6 +422,7 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		// problems or firewall restrictions.
 		deployment.Spec.Template.Spec.Containers[0].LivenessProbe.Handler.HTTPGet.Host = "localhost"
 		deployment.Spec.Template.Spec.Containers[0].ReadinessProbe.Handler.HTTPGet.Host = "localhost"
+		deployment.Spec.Template.Spec.DNSPolicy = corev1.DNSClusterFirstWithHostNet
 	}
 
 	// Fill in the default certificate secret name.
@@ -635,6 +641,15 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		)
 	}
 
+	if ci.Spec.HTTPHeaders != nil && len(ci.Spec.HTTPHeaders.HeaderNameCaseAdjustments) > 0 {
+		var adjustments []string
+		for _, v := range ci.Spec.HTTPHeaders.HeaderNameCaseAdjustments {
+			adjustments = append(adjustments, string(v))
+		}
+		v := strings.Join(adjustments, ",")
+		env = append(env, corev1.EnvVar{Name: RouterHTTPHeaderNameCaseAdjustments, Value: v})
+	}
+
 	if HTTP2IsEnabled(ci, ingressConfig) {
 		env = append(env, corev1.EnvVar{Name: RouterDisableHTTP2EnvName, Value: "false"})
 	} else {
@@ -896,6 +911,7 @@ func hashableDeployment(deployment *appsv1.Deployment, onlyTemplate bool) *appsv
 		return containers[i].Name < containers[j].Name
 	})
 	hashableDeployment.Spec.Template.Spec.Containers = containers
+	hashableDeployment.Spec.Template.Spec.DNSPolicy = deployment.Spec.Template.Spec.DNSPolicy
 	hashableDeployment.Spec.Template.Spec.HostNetwork = deployment.Spec.Template.Spec.HostNetwork
 	volumes := make([]corev1.Volume, len(deployment.Spec.Template.Spec.Volumes))
 	for i, vol := range deployment.Spec.Template.Spec.Volumes {
@@ -913,6 +929,10 @@ func hashableDeployment(deployment *appsv1.Deployment, onlyTemplate bool) *appsv
 		return volumes[i].Name < volumes[j].Name
 	})
 	hashableDeployment.Spec.Template.Spec.Volumes = volumes
+	hashableDeployment.Spec.Template.Annotations = make(map[string]string)
+	if val, ok := deployment.Spec.Template.Annotations[LivenessGracePeriodSecondsAnnotation]; ok && len(val) > 0 {
+		hashableDeployment.Spec.Template.Annotations[LivenessGracePeriodSecondsAnnotation] = val
+	}
 
 	if onlyTemplate {
 		return &hashableDeployment
@@ -1008,7 +1028,7 @@ func (r *reconciler) updateRouterDeployment(current, desired *appsv1.Deployment)
 	return true, nil
 }
 
-// deepHashObject writes specified object to hash using the spew library
+// deepHashObject writes a specified object to a hash using the spew library
 // which follows pointers and prints actual values of the nested objects
 // ensuring the hash does not change when a pointer changes.
 //
@@ -1025,7 +1045,7 @@ func deepHashObject(hasher hash.Hash, objectToWrite interface{}) {
 }
 
 // deploymentConfigChanged checks if current config matches the expected config
-// for the ingress controller deployment and if not returns the updated config.
+// for the ingress controller deployment and if it does not, returns the updated config.
 func deploymentConfigChanged(current, expected *appsv1.Deployment) (bool, *appsv1.Deployment) {
 	if deploymentHash(current) == deploymentHash(expected) {
 		return false, nil
@@ -1040,7 +1060,16 @@ func deploymentConfigChanged(current, expected *appsv1.Deployment) (bool, *appsv
 		containers[i+1] = *container.DeepCopy()
 	}
 	updated.Spec.Template.Spec.Containers = containers
+	updated.Spec.Template.Spec.DNSPolicy = expected.Spec.Template.Spec.DNSPolicy
 	updated.Spec.Template.Labels = expected.Spec.Template.Labels
+
+	if val, ok := expected.Spec.Template.Annotations[LivenessGracePeriodSecondsAnnotation]; ok && len(val) > 0 {
+		if updated.Spec.Template.Annotations == nil {
+			updated.Spec.Template.Annotations = make(map[string]string)
+		}
+		updated.Spec.Template.Annotations[LivenessGracePeriodSecondsAnnotation] = val
+	}
+
 	updated.Spec.Strategy = expected.Spec.Strategy
 	volumes := make([]corev1.Volume, len(expected.Spec.Template.Spec.Volumes))
 	for i, vol := range expected.Spec.Template.Spec.Volumes {
